#! /usr/local/bin/sl
#
#	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

### \section[NAME]
## htmlprep.sc, htp.sc -- a HTML preprocessor

### \section[SYNOPSIS]
## \em[htmlprep [filename [out-file]]] 
### \section[DESCRIPTION]
## HtmlPrep is a HTML preprocessor written in NSL (see \ref[sl(1)])
##
## It adds a set of tags to a standard HTML which allow dynamical generation
## of HTML pages. It allows a limited subset of \ref[sl(1)] 
## commands to be
## executed within HTML templates. HtmlPrep is transparent, i. e. any valid
## HTML document will pass thru it unchanged.
##
## It can be used as a CGI script (with no argument) or as a stand-alone
## tool (with a filename argument). It cannot however serve as a pipe,
## since it requires a seekable stream as the input. 
##
## In a stand-alone incarnation output is directed to \em[out-file] or
## stdout if the latter is not present

.module <postgres>
.module <files>
.pool "DIVERSIONS"
main
{
	if getenv(GATEWAY_INTERFACE)
		read
		def has_qparams = $[*];
		defstream ins getenv(PATH_TRANSLATED) html;
		write 'Content-Type: text/html'
		write ''
	else
		if not(check(ARGV 1)) error 'No input file given and not under CGI' endif
		defstream ins get(ARGV 1) html;
		defstream outs cond(check(ARGV 2) get(ARGV 2) '') html write;
		outstream outs
	endif
	getpos ins
	onerror error-handler
	ioctl ins keepspaces
	instream ins
	defautoseq escape_match escape:EscapeMatch
	defautoseq urlencode urlencode;
	defautoseq cvthtmlarg html;
	subload framework empty
	table storagemap ''
	global sto.uniq.counter = 1;
	defprocin framework rmstorage 'release storage $1 _rmstorage($1)'
	defprocin framework version 
		'if not($1) return(0.1) elsif eqi($1 nsl) return(version()) else return(version($1))'
	callin framework module files;
	callin framework module crypt;
	callin framework load loadsess;
	restrict framework
		- tag var arg eval return error iseof
		defined def readonly set push pop depth
		copy exch revstack isint add incr postincr
		mul sub decr postdecr div rem neg abs int
		hex dec oct eq eqi neq less greater le ge
		expr match matchi between and or not
		cond length ascii char trim substr
		index rindex indexi rindexi
		replace repeat split stosplit 
		span reverse substit expand
		get check put insert remove
		apply mark-remove unmark-remove ismark-remove
		cancel-remove do-remove count 
		getenv exists getcwd filesize isnewer ioerror clearerr 
		feature pid id crypt;
	embed framework proc embedded_table table
	embed framework proc embedded_array array
	embed framework proc embedded_list list
	embed framework proc embedded_sequence sequence
	embed framework proc embedded_iota iota
	embed framework proc embedded__rmstorage _rmstorage
	embed framework proc embedded_urlencode urlencode
	subload aux htmlprepaux
	import proc convert-html-param aux:convert-html-param
	global translator = '';
	global do_subst = true;
	global if_counter = 0;
	global loop_type;
	global loop_arg;
	global loop_counter;
	global loop_pos;
	global sqlquery;
	global stdout = outstream();
	global stdin  = instream()
	globalin framework :repeat;
	if defined(has_qparams)
		def var val;
		while $has_qparams
			set has_qparams = split($has_qparams '\t' '\n' var val)
			globalin framework url.$var = $val;
		loop
	endif
}	

embed_storage(intname name)
{
	if or(not($name) eq($name *))
		set name = '#sto.'pad(hex(postincr(sto.uniq.counter 1)) 8 0)
	endif
	insert storagemap $name $intname
	embed framework storage $intname $name
}

embedded_table(name arg)
{
	if and($arg neq(+ substr($arg 0 1)))
		if eq(! substr($arg 0 1))
			set arg = getenv(HTMLPREP_TABLES)/!substr($arg 1)
		else
			set arg = getenv(HTMLPREP_TABLES)/$arg
		endif
	endif
	def intname = table(* arg);
	embed_storage($intname $name)
}

embedded_array(name)
{
	embed_storage(array(*) $name)
}

embedded_list(arg)
{
	embed_storage(list($arg) *)
}

embedded_sequence(arg)
{
	embed_storage(sequence($arg) *)
}

embedded_iota(arg)
{
	embed_storage(iota($arg) *)
}

embedded__rmstorage(name)
{
	release storage get(storagemap $name)
	remove storagemap $name
}

embedded_urlencode(arg)
{
	return(translate(reset(urlencode) $arg))
}

tag "HTML" 
{
	if tag(USES)
		def pref;
		def name;
		def l;
		def i;
		set pref = split(tag(USES) ':' '' name);
		local table(params +$(name))
		set l = length($(pref).)
		if $pref
			set pref = translate(escape_match $pref).*
		else 
			set pref = '*'
			set l = 0
		endif
		foreach i params
			if match($pref $i)
				globalin framework substr($i $l) = get(params $i);
			endif
		loop
		rmtag USES
	endif
	writetag
}

eval-param(param)
{
	return(callin(framework eval convert-html-param(tag($param))))
}

tag "TRANSLATE"
{
	if tag(NONE)
		push(translator = '')
	elsif tag(SEQ)
		push(translator = defautoseq(* tag(SEQ)))
	endif
}

tag "/TRANSLATE"
{
	if $translator release autoseq $translator endif
	pop translator
}

tag "DEFINE"
{
	if tag(VAR)
		settag NAME tag(VAR)
	endif
	globalin framework tag(NAME) = eval-param(VALUE);
}

tag "SET"
{
	if tag(VAR)
		settag NAME tag(VAR)
	endif
	callin(framework set tag(NAME) = eval-param(VALUE))
}

tag "SQLCONNECT"
{
	safe-sql(connect callin(framework expand tag(TARGET)))
}

tag "/SQLCONNECT"
{
	safe-sql(disconnect);
}

fill-in-sql-fields
{
	def iter = sto.enum($sqlquery loop_arg);
	if not($loop_arg) return endif
	def row = local(get($sqlquery $iter));
	def field;
	cleartag
	settag '' count($sqlquery)
	foreach field $row
		settag $field get($row $field)
	loop
	passtag framework
	return(true)
}

tag "QUERY"
{
	def sq = safe-sql(query callin(framework expand tag(SQL)));
	if eq(int 0 count($sq))
		release storage $sq
		prefix 'false-loop:'
		eval-param(IFEMPTY)
		set loop_counter = 1
	else
		push(sqlquery = $sq)
		push(loop_type = query)
		push(loop_pos = getpos(ins))
		push loop_arg
		fill-in-sql-fields()
	endif
}

tag "NOSUBST" { set do_subst = '' }
tag "/NOSUBST" { set do_subst = true }

tag "IF"
{
	if not(eval-param(COND))
		prefix "false-if0:"
		set if_counter = 1
	endif
}

tag "false-if?:IF" { incr if_counter 1 }
tag "false-if?:ENDIF" { decr if_counter 1 
						if le(int $if_counter 0) prefix '' endif 
					  }
tag "false-if0:ELSE"
{
	if eq(int $if_counter 1)
		clear if_counter
		prefix ''
	endif
}
tag "false-if*" { }
char "false-if*" { }

tag "ELSE" { prefix "false-if1:" }

tag "ENDIF" {} 

tag "WHILE"
{
	if not(eval-param(COND))
		prefix "false-loop:"
		 set loop_counter = 1;
	else
		push(loop_type = while)
		push(loop_pos  = getpos(ins))
		push(loop_arg  = convert-html-param(tag(COND)))
	endif
}

tag "REPEAT"
{
	if le(int eval-param(COUNT) 0)
		prefix "false-loop:"
		set loop_counter = 1;
	else
		push(loop_type = repeat)
		push(loop_pos = getpos(ins))
		push(loop_arg = sub(tag(COUNT) 1))
		callin framework push :repeat = 1;
	endif
}

tag "FOREACH"
{
	def handler sto;
	def first = sto.enum(set(sto = get(storagemap eval-param(STORAGE))) handler);
	if not($handler)
		prefix "false-loop:"
		set loop_counter = 1
	else
		push(loop_type = foreach)
		push(loop_pos = getpos(ins))
		push(loop_arg = $sto\ tag(VAR)\ $handler)
		callin framework set tag(VAR) = $first;
	endif
}

pop_loop
{
	switch $loop_type
		case while
			breaksw
		case foreach
			if $loop_arg
				def sto;
				def h = split($loop_arg ' ' '' sto '');
				sto.break($sto h)
			endif
			breaksw
		case repeat
			callin framework pop :repeat;
			breaksw
		case query
			release storage $sqlquery
			pop sqlquery
			breaksw
	endsw
	pop loop_type
	pop loop_pos
	pop loop_arg
}

tag "LOOP"
{
	if tag(WHILE)
		if not(eval-param(WHILE))
			pop_loop()
			return
		endif
	endif
	switch $loop_type
		case while
			settag COND $loop_arg
			if eval-param(COND)
				setpos instream() $loop_pos
			else 
				pop_loop()
			endif
			breaksw
		case foreach
			def var sto h next;
			set h = split($loop_arg ' ' '' sto var)
			set next = sto.enum($sto h)
			if not($h)
				set loop_arg = ''
				pop_loop()
			else
				setpos instream() $loop_pos
				callin framework set $var = $next;
			endif
			breaksw
		case repeat
			if eq(int 0 $loop_arg)
				pop_loop()
			else
				setpos instream() $loop_pos
				decr loop_arg 1
				callin framework incr :repeat 1;
			endif
			breaksw
		case query
			if fill-in-sql-fields()
				setpos instream() $loop_pos
			else
				pop_loop()
			endif
			breaksw
	endsw
}

tag "LOOPBODY"
{
	if not($loop_type) error "LOOPBODY outside a loop" endif
	set loop_pos = getpos(ins)
}

tag "false-loop:WHILE" { incr loop_counter 1 }
tag "false-loop:REPEAT" { incr loop_counter 1 }
tag "false-loop:QUERY" { incr loop_counter 1 }
tag "false-loop:LOOP" { decr loop_counter 1 if eq(int $loop_counter 0) prefix '' endif }
tag "false-loop:*" { }
char "false-loop:?" { }


tag "EXECUTE" { 
	settag CODE '{'tag(CODE)'}'
	eval-param(CODE) 
}

tag "INSERT"
{
	if tag(RAW)
		ioctl outstream() raw
		write eval-param(VALUE)
		ioctl outstream() cooked
	elsif tag(DIVERT)
		if not(defined(diversion\.tag(DIVERT)))
			error "Diversion tag(DIVERT) not defined"
		endif
		ioctl outstream() raw
		write var(diversion\.tag(DIVERT))
		ioctl outstream() cooked
	else
		write translate($translator eval-param(VALUE))
	endif
}

eof
{
	def tmp = $stdout;
	pop stdout;
	if neq($stdout $tmp)
		rmstream $tmp
	endif
	if $stdout
		outstream $stdout
	endif
	rmstream $stdin
	pop stdin
	if $stdin
		instream $stdin
	endif
}

tag "INCLUDE"
{
	def tmp;
	using "DIVERSIONS"
	if tag(DIVERT)
		if not(defined(tag(DIVERT)))
			shared diversion\.tag(DIVERT);
		endif
		set tmp = defstream(* "inline://DIVERSIONS/diversion\.tag(DIVERT)" html true);
		outstream $tmp
		push(stdout = $tmp)
	else
		push(stdout = $stdout)
	endif
	set tmp = eval-param(FILE)
	if exists($tmp)
		set tmp = defstream(* "file://$tmp" html);
		ioctl $tmp keepspaces
		instream $tmp
		push(stdin = $tmp)
		if tag(RAW)
			ioctl $tmp raw
		endif
	endif
}

tag "FUNCTION"
{
	defprocin framework tag(NAME) tag(BODY)
}

tag "RESET" { reset $translator }

tag "." { if $translator write translatez($translator ''); endif }

tag "DNL" { prefix 'dnl:' }
 
tag "dnl:*" { }
char "dnl:?" { if eq($[value] \n) prefix '' endif }

tag "*" { 
	if tag(IF)
		if not(eval-param(IF)) return endif
		rmtag IF
	endif
	if $translator
		write translatez($translator '')
	endif
	if $do_subst
		def all = $[*];
		def name val;
		while $all
			set all = split($all '\t' '\n' name val);
			settag $name translate(reset(cvthtmlarg) translate(reset($translator) callin(framework expand $val)))
		loop($all)
		reset($translator)
	endif
	writetag
}

char "?" { write translate($translator $[value]); }


safe-sql
{
	def r = sql($1 $2);
	if(sql(error))
		error sql(error);
	endif
	if(sql(response))
		error sql(response)
	endif
	sql(clearerr);
	return($r);
}

error-handler(desc)
{
	echo "id(): Error occured at caller(id): $desc"
	exit
}
