#!/usr/local/bin/sl

#	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



main
{
#	trace(*)
	global base = get(ARGV 4);
	subload li linefilter
	defstream ins li?get(ARGV 1) script;
	defstream outs get(ARGV 2) html write;
	table refs docstrip_refs;
	instream ins
	outstream outs
	writetag HTML;
	write \n;
	writetag META NAME Generator CONTENT 'NSL/man2html';
	write \n
	if $base
		writetag BASE HREF $base;
		write \n
	endif
	writetag TITLE;
	write get(ARGV 3)
	writetag /TITLE;
	write \n
	write \n
	writetag BODY;
	write \n
	global par-end closing;
	global count = 0;
	echo 'Generating HTML from man'
}

eof
{
	write \n
	writetag /BODY;
	write \n
	writetag /HTML;
	write \n
	echo ''
}

tag "LINE"
{
	echo '['incr(count 1)'] '\r
	if eq(. substr($[value] 0 1))
		def cmd rest;
		set rest = split(substr($[value] 1) ' ' '' cmd)
		if not(isproc(cmd_$cmd)) return endif
		call cmd_$cmd trim($rest);
	else
		print-line($[value])
	endif
	if $closing 
		writetag $closing;
		write \n
		set closing = ''
	endif
}

cmd_SH(arg)
{
	end-paragraph()
	this-or-next-line($arg H2)
}

cmd_SS(arg)
{
	end-paragraph()
	this-or-next-line($arg H3)
}

cmd_TP(arg)
{
	end-paragraph()
	set closing = BLOCKQUOTE
	set par-end = /BLOCKQUOTE
}

cmd_IP(arg)
{
	set arg = get-arg($arg)
	if not($arg)
		end-paragraph()
		writetag BLOCKQUOTE;
		write \n
		set par-end = /BLOCKQUOTE
	else 
		if neq($par-end /UL) 
			end-paragraph() 
			writetag UL;
			write \n
		endif
		write \n
		writetag LI;
		if neq($arg '\\(bu')
			print-line($arg)
		endif
		set par-end = /UL
	endif
}

standard-paragraph()
{
	end-paragraph()
	write \n
	writetag P;
	write \n
}

cmd_HP { standard-paragraph() }
cmd_P { standard-paragraph() }
cmd_LP { standard-paragraph() }
cmd_PP { standard-paragraph() }

cmd_RS(arg)
{
	writetag BLOCKQUOTE;
	write \n
}

cmd_RE(arg)
{
	writetag /BLOCKQUOTE;
	write \n
}

cmd_SM(arg)
{
	this-or-next-line($arg SMALL)
}

cmd_SB(arg)
{
	this-or-next-line($arg BIG)
}

interlace-words(args font1 font2)
{
	def cur_font = $font1;
	while $args
		if $cur_font writetag $cur_font; endif
		set args = print-word($args)
		if $cur_font writetag /$cur_font; endif
		set cur_font = cond(eq($cur_font $font1) $font2 $font1)
	loop
	write \n
}

cmd_BI(args) { interlace-words($args STRONG EM) }
cmd_IB(args) { interlace-words($args EM STRONG) }
cmd_RI(args) { interlace-words($args '' EM) }
cmd_IR(args) { interlace-words($args EM '') }
cmd_RB(args) { interlace-words($args '' STRONG) }
cmd_BR(args) { interlace-words($args STRONG '') }

this-or-next-line(arg font)
{
	writetag $font;
	if $arg
		print-word($arg)
		writetag /$font;
		write \n
	else
		set closing = /$font;
	endif
}

cmd_R(arg)
{
	if $arg print-word($arg) endif
}

cmd_I(arg)
{
	this-or-next-line($arg EM)
}

cmd_B(arg)
{
	def href = get-arg($arg);
	if check(refs $href)
		if and($base not(index($href ://)))
			set href = $base/$href\.html
		else 
			set href = $href\.html
		endif
		set href = substit($href \( .)
		set href = substit($href \) '')
		writetag A HREF $href;
		print-word($arg)
		writetag /A;
	else
		this-or-next-line($arg STRONG)
	endif
}

print-line(line)
{
	while $line
		set line = print-char($line)
	loop
	write \n
}

print-word(line)
{
	def delim = ' ';
	while eq(' ' substr($line 0 1))
		set line = substr($line 1)
	loop
	if eq(substr($line 0 1) \")
		set delim = \"
		set line = substr($line 1)
	endif
	while $line
		if eq(substr($line 0 1) $delim)
			return(substr($line 1))
		endif
		set line = print-char($line)
	loop
	return($line)
}

print-args(line)
{
	while $line
		set line = print-word($line)
	loop
	write \n
}

get-arg(line)
{
	def arg;
	if eq(substr($line 0 1) \")
		split(substr($line 1) \" '' arg)
	else
		split($line ' ' '' arg)
	endif
	return($arg)
}

end-paragraph()
{
	if $par-end
		writetag $par-end;
		write \n
		set par-end = ''
	endif
}

print-char(ch)
{
	def name;
	if neq(\\ substr($ch 0 1))
		write substr($ch 0 1);
		return(substr($ch 1))
	else
		set ch = substr($ch 1)
		switch substr($ch 0 1)
			when eq($^ ' ')
				writetag & value nbsp;
				return(substr($ch 1))
			when eq($^ '(')
				set name = substr($ch 1 2)
				switch $name
					when eq($^ co)
						writetag & value copy;
						breaksw;
					when eq($^ rg)
						writetag & value reg;
						breaksw;
					when eq($^ cq)
						write \'
						breaksw
					when eq($^ rq)
						write \"
						breaksw
					when eq($^ rs)
						write \\
						breaksw
					when true
						write ???
				endsw
				return(substr($ch 3))
			when true
				write $^;
				return(substr($ch 1))
		endsw
	endif
}

