#
# 	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

.include <mailbox.sci>
.include <fileutil>

main
{
	array(months Jan Feb Mar Apr May Jun Jul Aug Oct Nov Dec)
	table identity get(ARGV 1)
	if not(exists(get(identity mbox))) error "$mbox doesn\'t exist" endif
	def mbox = get(identity mbox);
	def newlen = filesize($mbox) lastlen = get(identity lastlen);
	def curpos nextpos;
	if less(int $newlen $lastlen)
		set lastlen = 0
	elsif eq(int $newlen $lastlen)
		echo "nothing to do"
		exit
	endif
	if not(dotlock-wait($mbox 5)) error "cannot lock $mbox" endif
	global tstamp = cond(check(identity timestamp) get(identity timestamp));
	global touchfile = cond(check(identity touchfile) get(identity touchfile));
	put identity lastlen $newlen
	onerror error-handler
	defautoseq tolower ascii:Tolower
	defautoseq koi2win koi8r:Tounicode|cp1251:Tonative
	defautoseq win2koi cp1251:Tounicode|koi8r:Tonative
	defautoseq lf linefilter
	defstream mbox file://$mbox basic;
	export stream mbox mbox
	setmailbox(mbox)
	subload base64 base64enc
	export subint base64 base64
	set curpos = $lastlen
	def subject from myid = get(identity id) id passcode
		myemail = get(identity email) mypasscode = get(identity passcode);
	while $curpos
		echo "processing $curpos"
		set nextpos = nextmail($curpos)
		set from = get-email(headerfld($curpos from))
		set subject = trim(headerfld($curpos subject))
		split($subject ':' '' id passcode)
		if eq($id $myid)
			if not(eqi($from $myemail))
				notify-warning("attempt to access $id from $from")
			elsif neq($passcode $mypasscode)
				notify-warning("wrong passcode, msg headerfld($curpos message-id)")
			else
				process-message($curpos $nextpos)
			endif
		endif
		set curpos = $nextpos
	loop
	rmfile $mbox\.lock
}

process-message(pos limit)
{
	def type = headerfld($pos content-type)
		dispos = headerfld($pos content-disposition)
		fname
		majortype minortype;
	set type = split($type '/' ';' majortype minortype)
	set fname = split($dispos ';' '' dispos)
	echo "$majortype/$minortype"
	if eqi($majortype multipart)
		process-mixed($pos $minortype $type $limit)
		return
	endif
	if eqi($dispos attachment)
		set fname = gethdrparam($dispos filename)
		if not($fname)
			set fname = gethdrparam($type name)
		endif
	else
		set fname = ''
	endif
	if $fname
		process-attachement($pos $majortype $fname $limit)
	elsif eqi($majortype text)
		process-news($pos $type $limit)
	endif
}

process-mixed(pos minortype rest limit)
{
	def nextpos eof;
	def boundary = gethdrparam($rest boundary);
	echo "boundary=$boundary"
	set eof = endofpart(mailbody($pos) $boundary)
	set pos = nextpart($eof $boundary)
	if eqi($minortype alternative)
		process-message($pos $eof)
	else
		while $pos;
			set eof = endofpart($pos $boundary)
			set nextpos = nextpart($eof $boundary)
			process-message($pos $eof)
			set pos = $nextpos
		loop;
	endif
}

process-attachement(pos type fname limit)
{
	set fname = dosbasename(basename($fname))
	def dir;
	set type = translate(reset(tolower) $type)
	if check(identity $type)
		set dir = get(identity $type)
	elsif check(identity *)
		set dir = get(identity *)
	else
		notify-warning("unknown MIME type $type for $fname")
		return
	endif
	open-encoded-stream(instream $fname $pos)
	defstream outstream file://$dir/$fname basic true;
	copy-stream(instream outstream $limit)
	release stream instream
	release stream outstream
}

process-news(pos params limit)
{
	if not(check(identity news)) return endif
	def newsfile  = get(identity news)
		newsext  =  get(identity newsext)
		newscharset = get(identity newscharset)
		newsmax     = get(identity newsmax) tmp tmp1 pos1 date;
	if $tstamp
		set date = headerfld($pos date)
	endif
	open-encoded-stream(instream news $pos)
	set tmp = readfrom(instream lf)
	if isprefix(trim($tmp) --process)
		switch $tmp
			when set(pos1 index($tmp delete))
				set tmp = substr($tmp add($pos1 length(delete)))
				backshift-news-files($newsfile $newsext trim($tmp) $newsmax)
				touchfile()
				breaksw
			when set(pos1 index($tmp replace))
				set tmp = substr($tmp add($pos1 length(replace)))
				defstream outstream file://$(newsfile)\.trim($tmp)$newsext basic true
				if $date
					writeto outstream format-date($tstamp $date)
				endif
				copy-stream-trn(instream outstream $limit detect-translation(gethdrparam($params charset) $newscharset))
				release stream outstream
				touchfile()
				breaksw
			when set(pos1 index($tmp swap))
				set tmp = trim(substr($tmp add($pos1 length(swap))))
				split($tmp ':' '' tmp tmp1)
				mvfile $(newsfile)\.trim($tmp)$newsext $(newsfile)\.tmp$newsext
				mvfile $(newsfile)\.trim($tmp1)$newsext $(newsfile)\.trim($tmp)$newsext
				mvfile $(newsfile)\.tmp$newsext $(newsfile)\.trim($tmp1)$newsext
				touchfile()
				breaksw
			when set(pos1 index($tmp move))
				set tmp = trim(substr($tmp add($pos1 length(move))))
				split($tmp ':' '' tmp tmp1)
				move-news-files($newsfile $newsext trim($tmp) trim($tmp1) $newsmax)
				touchfile()
				breaksw
			when set(pos1 index($tmp get))
				set tmp = trim(substr($tmp add($pos1 length(move))))
				mail-me-article($tmp)
				breaksw
			when true
				if neq(trim($tmp) --process)
					notify-warning("Invalid request \'$tmp\'")
					breaksw
				endif
				shift-news-files($newsfile $newsext $newsmax)
				defstream outstream file://$newsfile\.0$newsext basic true
				if $date
					writeto outstream format-date($tstamp $date)
				endif
				copy-stream-trn(instream outstream $limit detect-translation(gethdrparam($params charset) $newscharset))
				release stream outstream
				touchfile()
		endsw
	endif
	release stream instream
}


shift-news-files(newsfile newsext newsmax)
{
	def i = $newsmax;
	while neq(int $i 0)
		if exists($newsfile\.sub($i 1)$newsext)
			mvfile $newsfile\.sub($i 1)$newsext $newsfile\.$i$newsext
		endif
		decr i 1
	loop
	rmfile $newsfile\.$newsmax$newsext
}

backshift-news-files(newsfile newsext newsmin newsmax)
{
	def i = $newsmin;
	rmfile $newsfile\.$i$newsext
	while less(int $i $newsmax)
		if exists($newsfile\.add($i 1)$newsext)
			mvfile $newsfile\.add($i 1)$newsext $newsfile\.$i$newsext
		endif
		incr i 1
	loop
}

move-news-files(newsfile newsext news1 news2 newsmax)
{
	mvfile $newsfile\.$news1$newsext $newsfile\.tmp$newsext
	def i = $news1;
	while less(int $i $newsmax)
		if exists($newsfile\.add($i 1)$newsext)
			mvfile $newsfile\.add($i 1)$newsext $newsfile\.$i$newsext
		endif
		incr i 1
	loop
	while neq(int $i $news2)
		if exists($newsfile\.sub($i 1)$newsext)
			mvfile $newsfile\.sub($i 1)$newsext $newsfile\.$i$newsext
		endif
		decr i 1
	loop
	rmfile $newsfile\.$newsmax$newsext
	mvfile $newsfile\.tmp$newsext $newsfile\.$news2$newsext
}

detect-translation(src-charset dest-charset)
{
	switch $src-charset
		when eqi($^ koi8r)
		when eqi($^ koi8-r)
		when eqi($^ koi8)
			return(cond(eq($dest-charset cp1251) koi2win))
		when eqi($^ windows-1251)
		when eqi($^ cp1251)
		when eqi($^ cp-1251)
			return(cond(eq($dest-charset koi8r) win2koi))
	endsw	
}

open-encoded-stream(name fname pos)
{
	def encoding = trim(headerfld($pos content-transfer-encoding));
	switch $encoding
		when eqi($^ 7bit)
		when eqi($^ 8bit)
		when eqi($^ binary)
		when not($^)
			defstream $name 'dup://mbox' basic;
			breaksw
		when eqi($^ base64)
			defstream $name 'script[base64]://dup://mbox' basic;
			breaksw
		when true
			notify-warning("unsupported encoding $encoding for $fname")
			return
	endsw
	setpos instream mailbody($pos)
}

copy-stream(in out limit)
{
	while;
		if and($limit ge(int getpos($in) $limit)) break endif
		readfrom($in)
		if iseof() break endif
		writetagto $out;
	loop;
}

copy-stream-trn(in out limit au)
{
	def str;
	while;
		if and($limit ge(int getpos($in) $limit)) break endif
		set str = readfrom($in $au)
		if iseof() break endif
		if $str writeto $out $str endif
		if $[] writetagto $out; endif
	loop;
}

get-email(str)
{
	def idx = index($str \<) email;
	if $idx
		split(substr($str add($idx 1)) '>' '' email)
		return($email)
	else
		return(trim($str))
	endif
}

mail-me-article(no)
{
	defstream mailer "pipe:///usr/sbin/sendmail get(identity email)" basic true;
	if check(identity myemail)
		writeto mailer "From: get(identity myemail)\n"
	endif
	writeto mailer "To: get(identity email)\n"
	writeto mailer "Subject: get(identity id):\n"
	writeto mailer '\n'
	def fname = get(identity news).$(no)get(identity newsext) line;
	if exists($fname)
		writeto mailer "--process replace $no\n"
		defstream article file://$fname basic;
		copy-stream(article mailer '')
		rmstream article
	else
		writeto mailer "Article $no doesn\'t exists\n";
	endif
	rmstream mailer
}

notify-warning(msg)
{
	defstream mailer "pipe:///usr/sbin/sendmail get(identity email)" basic true;
	writeto mailer "To: get(identity email)\n"
	writeto mailer "Subject: job processing failure (newsmaker)\n"
	writeto mailer '\n'
	writeto mailer $msg\n
	echo "$msg"
	rmstream mailer
}

error-handler(desc fname lno)
{
	rmfile get(identity mbox).lock
	notify-warning("error $desc at $lno")
	pass-error $desc
}

touchfile()
{
	if $touchfile
		rmstream(defstream(* file://$touchfile basic true))
	endif
}

format-date(fmt date)
{
	def tmp year day mname hm hour min month;
	if set(tmp index($date ,))
		set date = substr($date add($tmp 1))
	endif
	toksplit(' %i %n %i %N:%N' $date '' day mname year hour min)
	foreach month iota(12)
		if eqi($mname get(months sub($month 1)))
			break
		endif
	loop
	if less(int $year 70)
		incr year 2000
	elsif less(int $year 1000)
		incr year 1900
	endif
	set fmt = substit($fmt 'yyyy' $year first)
	set fmt = substit($fmt 'yy' substr($year 2) first)
	set fmt = substit($fmt 'mmm' $mname first)
	set fmt = substit($fmt 'mm' pad($month 2 0) first)
	set fmt = substit($fmt 'dd' pad($day 2 0) first)
	set fmt = substit($fmt 'HH' pad($hour 2 0) first)
	set fmt = substit($fmt 'MM' pad($min 2 0) first)
	return($fmt)
}
